package SHARYANTO::Utils;

use 5.010001;

our $VERSION = '0.73'; # VERSION

1;
# ABSTRACT: SHARYANTO's temporary namespace for various routines

__END__

=pod

=encoding UTF-8

=head1 NAME

SHARYANTO::Utils - SHARYANTO's temporary namespace for various routines

=head1 VERSION

This document describes version 0.73 of SHARYANTO::Utils (from Perl distribution SHARYANTO-Utils), released on 2014-05-10.

=head1 FAQ

=head2 What is this?

This distribution is a heterogenous collection of modules that will eventually
have their own proper distributions. See L<SHARYANTO> about this temporary
namespace.

=head1 SEE ALSO

L<SHARYANTO>

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/SHARYANTO-Utils>.

=head1 SOURCE

Source repository is at L<https://github.com/sharyanto/perl-SHARYANTO-Utils>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=SHARYANTO-Utils>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

Steven Haryanto <stevenharyanto@gmail.com>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2014 by Steven Haryanto.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
